@@ -20,7 +20,7 @@ class ApiError(Exception):
   error: str
   detail: str
 
-  def __str__(self) -> None:
+  def __str__(self) -> str:
     return f'{self.error}: {self.detail}'
 
 
@@ -98,6 +98,27 @@ class BaserowClient(BaseClient):
     response = self._request('POST', '/api/user/token-refresh/', json=payload).json()
     return databind.json.load(response['user'], User), response['token']
 
+  def create_user(
+    self,
+    name: str,
+    email: str,
+    password: str,
+    authenticate: bool = False,
+    group_invitation_token: t.Optional[str] = None,
+    template_id: t.Optional[int] = None
+  ) -> t.Tuple[User, t.Optional[str]]:
+
+    payload: t.Dict[str, t.Union[str, bool, int]] = {'name': name, 'email': email, 'password': password}
+    if authenticate:
+      payload['authenticate'] = authenticate
+    if group_invitation_token:
+      payload['group_invitation_token'] = group_invitation_token
+    if template_id:
+      payload['template_id'] = template_id
+
+    response = self._request('POST', '/api/user/', json=payload).json()
+    return databind.json.load(response['user'], User), response.get('token')
+
   def list_groups(self) -> t.List[PermissionedOrderedGroup]:
     response = self._request('GET', '/api/groups/').json()
     return databind.json.load(response, t.List[PermissionedOrderedGroup])
@@ -130,7 +151,7 @@ class BaserowClient(BaseClient):
     self,
     table_id: int,
     exclude: t.Optional[t.List[str]] = None,
-    filter: t.List[t.List[Filter]] = None,
+    filter: t.Optional[t.List[Filter]] = None,
     filter_type: t.Optional[FilterType] = None,
     include: t.Optional[t.List[str]] = None,
     order_by: t.Optional[t.List[str]] = None,
@@ -140,7 +161,7 @@ class BaserowClient(BaseClient):
     user_field_names: bool = False,
   ) -> Page[t.Dict[str, t.Any]]:
 
-    params = {}
+    params: t.Dict[str, t.Optional[str]] = {}
     if exclude is not None:
       params['exclude'] = ','.join(exclude)
     if filter is not None:
@@ -152,13 +173,13 @@ class BaserowClient(BaseClient):
     if order_by is not None:
       params['order_by'] = ','.join(order_by)
     if page is not None:
-      params['page'] = page
+      params['page'] = str(page)
     if search is not None:
       params['search'] = search
     if size is not None:
-      params['size'] = size
+      params['size'] = str(size)
     if user_field_names:
-      params['user_field_names'] = True
+      params['user_field_names'] = 'True'
 
     response = self._request('GET', f'/api/database/rows/table/{table_id}/', params=params).json()
     if page is None:
@@ -170,6 +191,12 @@ class BaserowClient(BaseClient):
       page + 1 if response['next'] else None,
       response['results'])
 
+  def create_database_table_row(self, table_id: int, record: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
+    return self._request('POST', f'/api/database/rows/table/{table_id}/', json=record).json()
+
+  def update_database_table_row(self, table_id: int, row_id: int, record: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
+    return self._request('PATCH', f'/api/database/rows/table/{table_id}/{row_id}/', json=record).json()
+
   def get_database_table_row(self, table_id: int, row_id: int) -> t.Dict[str, t.Any]:
     return self._request('GET', f'/api/database/rows/table/{table_id}/{row_id}/').json()
 
@@ -241,11 +268,10 @@ class BaserowClient(BaseClient):
     self,
     table_id: int,
     exclude: t.Optional[t.List[str]] = None,
-    filter: t.List[t.List[Filter]] = None,
+    filter: t.Optional[t.List[Filter]] = None,
     filter_type: t.Optional[FilterType] = None,
     include: t.Optional[t.List[str]] = None,
     order_by: t.Optional[t.List[str]] = None,
-    page: t.Optional[int] = None,
     search: t.Optional[str] = None,
     size: t.Optional[int] = None,
     user_field_names: bool = False,
